@@ -114,8 +114,10 @@ DevicesPtr find_devices(Options *op)
     DisplayDevicePtr pDisplayDevice;
     int i, j, n, count = 0;
     unsigned int mask, bit;
+    DeviceRec tmpDevice;
     NvCfgDeviceHandle handle;
     NvCfgDevice *devs = NULL;
+    NvCfgBool is_primary_device;
     char *lib_path;
     void *lib_handle;
 
@@ -128,6 +130,8 @@ DevicesPtr find_devices(Options *op)
     NvCfgBool (*__getEDID)(NvCfgDeviceHandle handle,
                            unsigned int display_device,
                            NvCfgDisplayDeviceInformation *info);
+    NvCfgBool (*__isPrimaryDevice)(NvCfgDeviceHandle handle,
+                                  NvCfgBool *is_primary_device);
     NvCfgBool (*__closeDevice)(NvCfgDeviceHandle handle);
     
     /* dlopen() the nvidia-cfg library */
@@ -157,7 +161,8 @@ DevicesPtr find_devices(Options *op)
         dlclose(lib_handle);                                   \
         return NULL;                                           \
     }
-        
+
+    /* required functions */
     __GET_FUNC(__getDevices, "nvCfgGetDevices");
     __GET_FUNC(__openDevice, "nvCfgOpenDevice");
     __GET_FUNC(__getNumCRTCs, "nvCfgGetNumCRTCs");
@@ -165,6 +170,9 @@ DevicesPtr find_devices(Options *op)
     __GET_FUNC(__getDisplayDevices, "nvCfgGetDisplayDevices");
     __GET_FUNC(__getEDID, "nvCfgGetEDID");
     __GET_FUNC(__closeDevice, "nvCfgCloseDevice");
+
+    /* optional functions */
+    __isPrimaryDevice = dlsym(lib_handle, "nvCfgIsPrimaryDevice");
     
     if (__getDevices(&count, &devs) != NVCFG_TRUE) {
         return NULL;
@@ -232,6 +240,14 @@ DevicesPtr find_devices(Options *op)
             pDevices->devices[i].displayDevices = NULL;
         }
 
+        if ((i != 0) && (__isPrimaryDevice != NULL) &&
+            (__isPrimaryDevice(handle, &is_primary_device) == NVCFG_TRUE) &&
+            (is_primary_device == NVCFG_TRUE)) {
+            memcpy(&tmpDevice, &pDevices->devices[0], sizeof(DeviceRec));
+            memcpy(&pDevices->devices[0], &pDevices->devices[i], sizeof(DeviceRec));
+            memcpy(&pDevices->devices[i], &tmpDevice, sizeof(DeviceRec));
+        }
+        
         if (__closeDevice(handle) != NVCFG_TRUE)
             goto fail;
     }
@@ -299,13 +315,12 @@ static int set_xinerama(int xinerama_enabled, XConfigPtr config)
     }
 
     if (config->flags->options) {
-        remove_option_from_list(&(config->flags->options), "Xinerama");
+        xconfigRemoveNamedOption(&(config->flags->options), "Xinerama", NULL);
     }
 
-    config->flags->options =
-        xconfigAddNewOption(config->flags->options,
-                            nvstrdup("Xinerama"),
-                            nvstrdup(xinerama_enabled?"1":"0"));
+    xconfigAddNewOption(&config->flags->options,
+                        "Xinerama",
+                        (xinerama_enabled ? "1" : "0"));
 
     return TRUE;
 
@@ -477,8 +492,7 @@ static int enable_separate_x_screens(Options *op, XConfigPtr config,
      * new adjacencies
      */
     
-    xconfigFreeAdjacencyList(layout->adjacencies);
-    layout->adjacencies = NULL;
+    xconfigFreeAdjacencyList(&layout->adjacencies);
     
     create_adjacencies(op, config, layout);
 
@@ -604,7 +618,7 @@ static int disable_separate_x_screens(Options *op, XConfigPtr config,
                 }
                 next = screen->next;
                 screen->next = NULL;
-                xconfigFreeScreenList(screen);
+                xconfigFreeScreenList(&screen);
                 screen = next;
             } else {
                 
@@ -620,8 +634,7 @@ static int disable_separate_x_screens(Options *op, XConfigPtr config,
     
     /* wipe the existing adjacencies and recreate them */
     
-    xconfigFreeAdjacencyList(layout->adjacencies);
-    layout->adjacencies = NULL;
+    xconfigFreeAdjacencyList(&layout->adjacencies);
     
     create_adjacencies(op, config, layout);
 
@@ -813,17 +826,10 @@ static int enable_all_gpus(Options *op, XConfigPtr config,
     
     /* free all existing X screens, monitors, devices, and adjacencies */
     
-    xconfigFreeScreenList(config->screens);
-    config->screens = NULL;
-    
-    xconfigFreeDeviceList(config->devices);
-    config->devices = NULL;
-    
-    xconfigFreeMonitorList(config->monitors);
-    config->monitors = NULL;
-
-    xconfigFreeAdjacencyList(layout->adjacencies);
-    layout->adjacencies = NULL;
+    xconfigFreeScreenList(&config->screens);
+    xconfigFreeDeviceList(&config->devices);
+    xconfigFreeMonitorList(&config->monitors);
+    xconfigFreeAdjacencyList(&layout->adjacencies);
 
     /* add N new screens; this will also add device and monitor sections */
     
@@ -879,7 +885,7 @@ static void free_unused_devices(Options *op, XConfigPtr config)
             }
             next = device->next;
             device->next = NULL;
-            xconfigFreeDeviceList(device);
+            xconfigFreeDeviceList(&device);
             device = next;
         } else {
             prev = device;
@@ -923,7 +929,7 @@ static void free_unused_monitors(Options *op, XConfigPtr config)
             }
             next = monitor->next;
             monitor->next = NULL;
-            xconfigFreeMonitorList(monitor);
+            xconfigFreeMonitorList(&monitor);
             monitor = next;
         } else {
             prev = monitor;
@@ -945,13 +951,11 @@ static int only_one_screen(Options *op, XConfigPtr config,
     
     /* free all existing X screens after the first */
     
-    xconfigFreeScreenList(config->screens->next);
-    config->screens->next = NULL;
+    xconfigFreeScreenList(&config->screens->next);
     
     /* free all adjacencies */
     
-    xconfigFreeAdjacencyList(layout->adjacencies);
-    layout->adjacencies = NULL;
+    xconfigFreeAdjacencyList(&layout->adjacencies);
     
     /* add new adjacency */
     
