@@ -59,7 +59,7 @@ static void print_version(void)
     fmtoutp(TAB, "This program is used to manipulate X configuration files, "
             "specifically to enable NVIDIA X driver functionality.");
     fmtout("");
-    fmtoutp(TAB, "Copyright (C) 2005 NVIDIA Corporation.");
+    fmtoutp(TAB, "Copyright (C) 2005 - 2008 NVIDIA Corporation.");
     fmtout("");
     
 } /* print_version() */
@@ -612,6 +612,8 @@ void parse_commandline(Options *op, int argc, char *argv[])
             }
             break;
 
+        case PRESERVE_DRIVER_NAME_OPTION: op->preserve_driver = TRUE; break;
+        
         case DISABLE_SCF_OPTION: op->disable_scf = TRUE; break;
         
         case QUERY_GPU_INFO_OPTION: op->query_gpu_info = TRUE; break;
@@ -1016,7 +1018,7 @@ static XConfigPtr find_system_xconfig(Options *op)
     /* Sanitize the X config file */
     
     if (!xconfigSanitizeConfig(config, op->screen, &(op->gop))) {
-        xconfigFreeConfig(config);
+        xconfigFreeConfig(&config);
         return NULL;
     }
 
